
package org.slf4j.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * A simple (and direct) implementation that logs messages on the console
 * (<code>System.err<code>).
 *
 * <p>The output includes the relative time in milliseconds, thread
 * name, the level, logger name, and the message followed by the line
 * separator for the host.  In log4j terms it amounts to the "%r [%t]
 * %level %logger - %m%n" pattern. </p>
 *
 * <p>Sample output follows.</p>
 * <pre>
 * 176 [main] INFO examples.Sort - Populating an array of 2 elements in reverse order.
 * 225 [main] INFO examples.SortAlgo - Entered the sort method.
 * 304 [main] INFO examples.SortAlgo - Dump of integer array:
 * 317 [main] INFO examples.SortAlgo - Element [0] = 0
 * 331 [main] INFO examples.SortAlgo - Element [1] = 1
 * 343 [main] INFO examples.Sort - The next log statement should be an error message.
 * 346 [main] ERROR examples.SortAlgo - Tried to dump an uninitialized array.
 * at org.log4j.examples.SortAlgo.dump(SortAlgo.java:58)
 * at org.log4j.examples.Sort.main(Sort.java:64)
 * 467 [main] INFO  examples.Sort - Exiting main method.
 * </pre>
 *
 * @author Konstantinos Karadamoglou
 */
public class JDK11Logger extends MarkerIgnoringBase {
    /**
     * Mark the time when this class gets loaded into memory.
     */
    private static long startTime = System.currentTimeMillis();
    public static SimpleDateFormat CONSOLE_LOGGER_FORMAT = new SimpleDateFormat("HH:mm:ss.SSS",Locale.UK);
    public static final String LINE_SEPARATOR =
            System.getProperty("line.separator");
    String name;
    
    public static final String CONF_FILENAME="simple-log.properties";
    private static final String LEVEL_CONF_STRING="level";
    private static JDK11Level logLevel= JDK11Level.INFO;
    
    /**
     * Gets access to the simple-log.properties file and sets some getters of the
     * object. It actually loads the configuration of the logger.
     */
    static{
    	InputStream fin=JDK11Logger.class.getResourceAsStream("/"+CONF_FILENAME);
        if(fin!=null){
            Properties props=new Properties();
            
            try{
                props.load(fin);
            }catch(IOException ex){
                System.out.println("WARN: unable to load configuration file, fallback to the default settings");
            }
            
            JDK11Level parsedLevel=JDK11Level.getLevel(props.getProperty(LEVEL_CONF_STRING));
            if(parsedLevel!=null){
                logLevel=parsedLevel;
                System.out.println("INFO: JDK11Logger initialised successfully");
            }else{
                System.out.println("WARN: wrong configuration at property: "+LEVEL_CONF_STRING+" , fallback to the default settings");
            }
        }else{
            System.out.println("WARN: "+CONF_FILENAME+" not found, fallback to the default settings");
        }
        
    }
    
    /**
     * Package access allows only {@link SimpleLoggerFactory} to instantiate
     * SimpleLogger instances.
     */
    JDK11Logger(String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    
    public boolean isDebugEnabled() {
        return logLevel.isEqualAndLower(JDK11Level.DEBUG);
    }
    
    public void debug(String msg) {
        if(this.isDebugEnabled())
            log(JDK11Level.DEBUG, msg, null);
    }
    
    public void debug(String format, Object param1) {
        if(this.isDebugEnabled())
            formatAndLog(JDK11Level.DEBUG, format, param1, null);
    }
    
    public void debug(String format, Object param1, Object param2) {
        if(this.isDebugEnabled())
            formatAndLog(JDK11Level.DEBUG, format, param1, param2);
    }
    
    public void debug(String format, Object[] argArray) {
        if(this.isDebugEnabled())
            formatAndLog(JDK11Level.DEBUG, format, argArray);
    }
    
    public void debug(String msg, Throwable t) {
        if(this.isDebugEnabled())
            log(JDK11Level.DEBUG, msg, t);
    }
    
    /**
     * This is our internal implementation for logging regular (non-parameterized)
     * log messages.
     *
     * @param level
     * @param message
     * @param t
     */
    private void log(JDK11Level level, String message, Throwable t) {
        StringBuffer buf = new StringBuffer();
        
        buf.append(level.getName());
        buf.append(" ");
        
        buf.append(CONSOLE_LOGGER_FORMAT.format(new Date(System.currentTimeMillis())));
        buf.append(" ");
        
        buf.append(name);
        buf.append(" - ");
        
        buf.append(message);
        
        buf.append(LINE_SEPARATOR);
        
        System.err.print(buf.toString());
        if (t != null) {
            t.printStackTrace(System.err);
        }
        System.err.flush();
    }
    
    /**
     * For formatted messages, first substitute arguments and then log.
     *
     * @param level
     * @param format
     * @param param1
     * @param param2
     */
    private void formatAndLog(
            JDK11Level level, String format, Object arg1, Object arg2) {
        String message = MessageFormatter.format(format, arg1, arg2);
        log(level, message, null);
    }
    
    /**
     * For formatted messages, first substitute arguments and then log.
     *
     * @param level
     * @param format
     * @param argArray
     */
    private void formatAndLog(JDK11Level level, String format, Object[] argArray) {
        String message = MessageFormatter.arrayFormat(format, argArray);
        log(level, message, null);
    }
    
    public boolean isInfoEnabled() {
        return logLevel.isEqualAndLower(JDK11Level.INFO);
    }
    
    /**
     * A simple implementation which always logs messages of level INFO according
     * to the format outlined above.
     */
    public void info(String msg) {
        if(this.isInfoEnabled())
            log(JDK11Level.INFO, msg, null);
    }
    
    /**
     * Perform single parameter substitution before logging the message of level
     * INFO according to the format outlined above.
     */
    public void info(String format, Object arg) {
        if(this.isInfoEnabled())
            formatAndLog(JDK11Level.INFO, format, arg, null);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * INFO according to the format outlined above.
     */
    public void info(String format, Object arg1, Object arg2) {
        if(this.isInfoEnabled())
            formatAndLog(JDK11Level.INFO, format, arg1, arg2);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * INFO according to the format outlined above.
     */
    public void info(String format, Object[] argArray) {
        if(this.isInfoEnabled())
            formatAndLog(JDK11Level.INFO, format, argArray);
    }
    
    /**
     * Log a message of level INFO, including an exception.
     */
    public void info(String msg, Throwable t) {
        if(this.isInfoEnabled())
            log(JDK11Level.INFO, msg, t);
    }
    
    public boolean isWarnEnabled() {
        return logLevel.isEqualAndLower(JDK11Level.WARN);
    }
    
    /**
     * A simple implementation which always logs messages of level WARN according
     * to the format outlined above.
     */
    public void warn(String msg) {
        if(this.isWarnEnabled())
            log(JDK11Level.WARN, msg, null);
    }
    
    /**
     * Perform single parameter substitution before logging the message of level
     * WARN according to the format outlined above.
     */
    public void warn(String format, Object arg) {
        if(this.isWarnEnabled())
            formatAndLog(JDK11Level.WARN, format, arg, null);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * WARN according to the format outlined above.
     */
    public void warn(String format, Object arg1, Object arg2) {
        if(this.isWarnEnabled())
            formatAndLog(JDK11Level.WARN, format, arg1, arg2);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * WARN according to the format outlined above.
     */
    public void warn(String format, Object[] argArray) {
        if(this.isWarnEnabled())
            formatAndLog(JDK11Level.WARN, format, argArray);
    }
    
    /**
     * Log a message of level WARN, including an exception.
     */
    public void warn(String msg, Throwable t) {
        if(this.isWarnEnabled())
            log(JDK11Level.WARN, msg, t);
    }
    
    public boolean isErrorEnabled() {
        return logLevel.isEqualAndLower(JDK11Level.ERROR);
    }
    
    /**
     * A simple implementation which always logs messages of level ERROR according
     * to the format outlined above.
     */
    public void error(String msg) {
        if(this.isErrorEnabled())
            log(JDK11Level.ERROR, msg, null);
    }
    
    /**
     * Perform single parameter substitution before logging the message of level
     * ERROR according to the format outlined above.
     */
    public void error(String format, Object arg) {
        if(this.isErrorEnabled())
            formatAndLog(JDK11Level.ERROR, format, arg, null);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * ERROR according to the format outlined above.
     */
    public void error(String format, Object arg1, Object arg2) {
        if(this.isErrorEnabled())
            formatAndLog(JDK11Level.ERROR, format, arg1, arg2);
    }
    
    /**
     * Perform double parameter substitution before logging the message of level
     * ERROR according to the format outlined above.
     */
    public void error(String format, Object[] argArray) {
        if(this.isErrorEnabled())
            formatAndLog(JDK11Level.ERROR, format, argArray);
    }
    
    
    /**
     * Log a message of level ERROR, including an exception.
     */
    public void error(String msg, Throwable t) {
        if(this.isErrorEnabled())
            log(JDK11Level.ERROR, msg, t);
    }
    
}
